      PROGRAM BILCAL
C ***************************************************************
C *  BY  K. ERIC LIVO
C *
C *  PROGRAM CALIBRATES BIL FILES using an ASCII TEXT FILE
C *
C *  Input Calibration File Format:  ASCII Text, 1 text line
C *  per channel (band), each line containing two real variables:
C *  a multipling constant, then an additive constant, separated
C *  by a space.
C *  Text lines start with channel 1 and end with the last
C *  channel in the BIL file.
C *  Values in - range: (Raw Data * mult constant) + add constant
C *  = 0 to 1.00
C *  Values out - range: I*2  0 - 20,000  (0 - 100% reflectance)
C *
C *
C *  LINK WITH DISKIO, UNPACK
C *
C ***************************************************************
C
      INTEGER*4 IFCBIN(256),IFCBOUT(256),I,J,CODE
      INTEGER*2 IBUF(8192)
      REAL*8 RMULT(256), RADD(256)
      CHARACTER*80 FILENAME
C
C  LOGIT
C
      CALL REMLOG('********   BILCAL   ********')
C
C  OPEN REMAPP INPUT BIL FILE
C
      IFCBIN(1)=0
      CALL DISKIO(0,10,IBUF,IFCBIN)
      IF (IFCBIN(1) .LT. 0) CALL EXIT(3)
C
C  OPEN ASCII CALIBRATION CONSTANTS FILE
C
      WRITE(6,7)
7     FORMAT(1X,'ASCII CALIBRATION CONSTANTS FILENAME = '\)
      READ(5,8) FILENAME
8     FORMAT(80A)
      OPEN(11,ACCESS='SEQUENTIAL', FORM='FORMATTED',MODE='READ',
     &FILE=FILENAME,IOSTAT=CODE,STATUS='OLD')
      IF(CODE .NE. 0) STOP 'CALIBRATION FILENAME I/O ERROR'
C
C  OPEN CALIBRATED BIL OUTPUT FILE
C
      IFCBOUT(1)=16
      IFCBOUT(2)=IFCBIN(2)
      IF(IFCBIN(22).GT.1) THEN
         IFCBOUT(13) = 1
      ELSE
         IFCBOUT(13)=IFCBIN(13)
      ENDIF
      WRITE(6,5)
5     FORMAT(1X,'ENTER CALIBRATED BIL OUTPUT FILENAME')
      CALL DISKIO(0,12,IBUF,IFCBOUT)
C
C  READ CAL CONST ASCII FILE INTO ARRAY AND CONVERT TO 0 -20,000
C
      DO 10 I=1, IFCBOUT(13)
         READ(11,*,ERR=998) RMULT(I), RADD(I)
         RMULT(I)=RMULT(I)*20000.0
         IF (RMULT(I) .GT. 20000.0) RMULT(I)=20000.0
         RADD(I)=RADD(I)*20000.0
         IF (RADD(I) .GT. 20000.0) RADD(I)=20000.0
         WRITE(6,*) RMULT(I), RADD(I)
10    CONTINUE
C
C  READ SCANLINE SPECTRA SET
C
20    DO 30 I=1,IFCBOUT(13)
         CALL DISKIO(9,10,IBUF,IFCBIN)
         IF (IFCBIN(1) .EQ. -1) GOTO 999
         IF (IFCBIN(1) .LT. -1) CALL EXIT(3)
         IF (IFCBIN(25) .EQ. 8) CALL UNPACK(IBUF,IFCBIN)
         DO 40 J=1, IFCBIN(2)
            IBUF(J) = INT((IBUF(J)*RMULT(I)) + RADD(I))
            IF(IBUF(J).LT.0) IBUF(J)=0
            IF(IBUF(J).GT.20000) IBUF(J)=20000
40       CONTINUE
         CALL DISKIO(10,12,IBUF,IFCBOUT)
         IF (IFCBOUT(1) .LT. 0) CALL EXIT(3)
30    CONTINUE
      GOTO 20
C
C  ERROR READING ASCII CALIBRATION FILE
C
998   WRITE(6,995)
995   FORMAT(1X,'ERROR READING ASCII CALIBRATION FILE')
      STOP
C
C  CLOSE ALL FILES
C
999   CALL DISKIO(6,10,IBUF,IFCBIN)
      CLOSE(11)
      CALL DISKIO(6,12,IBUF,IFCBOUT)
      STOP
      END
