      PROGRAM CALFILE
C  *********************************************************
C  *  By K. Eric Livo                                      *
C  *                                                       *
C  *  PROGRAM generates ASCII output calibration table     *
C  *                                                       *
C  *  Divides calibrated absolute reflectance lab sample   * 
C  *  (site) ASCII spectra file (values 0 - 1) by          *
C  *  raw imaging spectrometer file pixel                  * 
C  *  (pixels summed if site larger than one pixel)        *
C  *                                                       *
C  *  outputs ASCII (scentific notation - E format)        *
C  *  multiplier and addition constants, 1 set per band    *
C  *      (addition constant always set to 0)              *
C  *  ASCII output file is to be multiplied into the raw   *
C  *  imaging spectrometry file to generate calibrated     * 
C  *  output.                                              *
C  *                                                       *
C  *  ASCII INPUT - per band, 2 real numbers ASCII:        *
C  *  abs. lab sample: wavelength, reflectance             *
C  *  (wavelength variable read, but ignored)              *
C  *                                                       *
C  *  all bands of BIL used - 1st band calculated          * 
C  *  automatically so:                                    * 
C  *  EXAMPLE - all bands may be entered:                  *
C  *                                    0 0 0 -1           *
C  *                                    0 0 0              *
C  *  or single band of image entered:                     *
C  *                                    0 0 0 20           *
C  *                                    0 0 0              *
C  *                                                       *
C  *  all bands with skips, image contains                 * 
C  *  224 BANDS with 2 scanline skip:                      *
C  *  user may enter data in full file  0 0 671 -1         *
C  *                           format.  0 0 2              *
C  *                                                       *
C  *  or single band of image with                         *    
C  *  2 scanline skip:                  0 0 2 20           *
C  *                                    0 0 2              *
C  *                                                       *
C  *  SPECIAL CASE: SCANLINE and PIXEL skips IDENTICAL     *
C  *  (skips=2)                         0 0 2 -1           *
C  *                                    0 0 2              *
C  *                                                       *
C  *  with subsetting if desired.                          *
C  *  Full BIL (fcb(13) file will be used, ONLY.           *
C  *                                                       *
C  *  3/10/91                                              *
C  *                                                       *
C  *  LINK WITH DISKIO, UNPACK                             *
C  *********************************************************
C
      INTEGER*4 INFCB(256), I, J, N, SL(1024), PIX(1024), PIXSUM(1024)
      INTEGER*4 FCBTEMP, CODE, FOFFSET, SKIPOFF, SREC
      INTEGER*2 IBUF(8192)
      REAL*4 MULT(1024), ADD(1024), RLABW(1024), RLABR(1024)
      CHARACTER*80 FILENAME
C
C  LOGIT
C
      CALL REMLOG('***   CALFILE.FOR   ***')
C
C  OPEN INPUT IMAGING SPECTROMETRY DATA
C
      CALL DISKIO(0,10,IBUF,INFCB)
      IF(INFCB(1).LT.0) CALL EXIT(3)
C
C  INPUT SCANLINE AND PIXEL SETS
C
      N=1
      WRITE(4,*) 'SCANLINE, PIXEL COORDINATE CALIBRATION SITE'
90    WRITE(6,100)
100   FORMAT
     &(1X,'ENTER IMAGE SCANLINE, PIXEL COORDINATE (0 0 - ENDS LOOP) '\)
      READ(5,*) SL(N), PIX(N)
      IF((SL(N).EQ.0).OR.(PIX(N).EQ.0)) GOTO 200
      WRITE(4,*) SL(N), PIX(N)
      N=N+1
      GOTO 90
C
C  OPEN LAB SPECTRA INPUT FILE
C
200   N=N-1
      IF(N.EQ.0) STOP 'NO calibration sites entered'
      WRITE(6,210)
210   FORMAT(1X,'LAB SPECTRA INPUT FILENAME = '\)
      READ(5,220) FILENAME
220   FORMAT(80A)
      OPEN(11,ACCESS='SEQUENTIAL',FORM='FORMATTED',MODE='READ',
     & FILE=FILENAME,IOSTAT=CODE,STATUS='OLD')
      IF(CODE.NE.0) STOP 'OPEN I/O ERROR'
C
C  READ LAB SPECTRA
C
      DO 240 J=1,INFCB(13)
         READ(11,*,ERR=998) RLABW(J), RLABR(J)
240   CONTINUE
C
C  OPEN ASCII OUTPUT FILE - E FORMAT
C
      WRITE(6,250)
250   FORMAT(1X,'ASCII CALIBRATION OUTPUT FILE = '\)
      READ(5,220) FILENAME
      OPEN(12,ACCESS='SEQUENTIAL',FORM='FORMATTED',
     & FILE=FILENAME,IOSTAT=CODE,STATUS='NEW')
      IF(CODE.NE.0) STOP 'OPEN I/O ERROR'
C
C  CALC CALIBRATION FILE, SCANLINE ALWAYS STARTS AT BAND 1
C    (PROGRAM CALC'S)
C
      DO 290 J=1,INFCB(13)
         PIXSUM(J) = 0
290   CONTINUE
      FOFFSET = INT((INFCB(5)-1)/INFCB(13))*INFCB(13)+1
      IF(INFCB(22).GT.1) THEN
         SKIPOFF = INFCB(7)
      ELSE
         IF(INFCB(6).EQ.INFCB(7)) THEN
            SKIPOFF = INFCB(13) * (INFCB(7) + 1) - 1
         ELSE
            SKIPOFF = INFCB(7)
         ENDIF
      ENDIF
      FCBTEMP = INFCB(5)
      INFCB(5) = FOFFSET
      DO 300 I=1,N
         SREC = FOFFSET + ((SL(I)-1)*(SKIPOFF+1))
         DO 310 J=1,INFCB(13)
            INFCB(14)=SREC + (J-1)
            CALL DISKIO(9,10,IBUF,INFCB)
            IF(INFCB(1) .EQ. -1) GOTO 999
            IF(INFCB(1) .LT. -1) CALL EXIT(4)
            IF(INFCB(25) .EQ. 8) CALL UNPACK(IBUF,INFCB)
            PIXSUM(J) = PIXSUM(J) + IBUF(PIX(I))
310      CONTINUE
300   CONTINUE
      INFCB(5) = FCBTEMP
C
      DO 400 J=1,INFCB(13)
         IF((PIXSUM(J).EQ.0).OR.(N.EQ.0)) THEN
            MULT(J) = 0.
         ELSE
            MULT(J) = RLABR(J)/(REAL(PIXSUM(J))/REAL(N))
         ENDIF
         ADD(J) = 0.
         WRITE(12,390,ERR=990) MULT(J), ADD(J)
400   CONTINUE
390   FORMAT(2E21.8)
      GOTO 999
C
C  ERROR MESSAGES
C
990   WRITE(4,991)
      WRITE(6,991)
991   FORMAT(1X,'ERROR WRITING ASCII OUTPUT CALIBRATION FILE')
      STOP
C
998   WRITE(4,995)
      WRITE(6,995)
995   FORMAT(1X,'ERROR READING ASCII LAB SPECTRA')
      STOP
C
C  FINISH
C
999   CLOSE(12)
      CALL DISKIO(6,10,IBUF,INFCB)
      CLOSE(11)
      STOP
      END
